<?php
declare(strict_types=1);

namespace CakeProducts\Controller;

use Cake\Datasource\Exception\RecordNotFoundException;
use Cake\Http\Response;
use Cake\Log\Log;
use CakeProducts\Controller\AppController;
use CakeProducts\Model\Enum\ProductCategoryAttributeTypeId;
use CakeProducts\Model\Table\ProductCategoryAttributesTable;
use CakeProducts\Service\CatalogManagerService;

/**
 * ProductCategoryAttributes Controller
 *
 * @property ProductCategoryAttributesTable $ProductCategoryAttributes
 */
class ProductCategoryAttributesController extends AppController
{
    /**
     * Index method
     *
     * @return Response|null|void Renders view
     */
    public function index()
    {
        $query = $this->ProductCategoryAttributes->find()
            ->contain(['ProductCategories']);
        $productCategoryAttributes = $this->paginate($query);

        $this->set(compact('productCategoryAttributes'));
    }

    /**
     * View method
     *
     * @param string|null $id Product Category Attribute id.
     * @return Response|null|void Renders view
     * @throws RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
        $productCategoryAttribute = $this->ProductCategoryAttributes->get($id, contain: [
            'ProductCategories',
            'ProductCategoryAttributeOptions',
        ]);
        $this->set(compact('productCategoryAttribute'));
    }

    /**
     * Add method
     *
     * @return Response|null|void Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
        $productCategoryAttribute = $this->ProductCategoryAttributes->newEmptyEntity();
        if ($this->request->is('post')) {
            $postData = $this->request->getData();
            if ($this->request->getSession()->read('Auth.User.id')) {
                $postData['created_by'] = $this->request->getSession()->read('Auth.User.id');
            }
            Log::debug(print_r('$postData', true));
            Log::debug(print_r($postData, true));
            $saveOptions = [
                'associated' => [
                    'ProductCategoryAttributeOptions'
                ],
            ];
            $productCategoryAttribute = $this->ProductCategoryAttributes->patchEntity($productCategoryAttribute, $postData, $saveOptions);
            if ($productCategoryAttribute->getErrors()) {
                Log::debug(print_r('$productCategoryAttribute->getErrors() next - failed to save from create new product category attribute', true));
                Log::debug(print_r($productCategoryAttribute->getErrors(), true));
            }
            if ($this->ProductCategoryAttributes->save($productCategoryAttribute, $saveOptions)) {
                $this->Flash->success(__('The product category attribute has been saved.'));

                return $this->redirect(['action' => 'index']);
            }
            Log::debug('failed to save new product category attribute errors next');
            Log::debug(print_r('$productCategoryAttribute->getErrors()', true));
            Log::debug(print_r($productCategoryAttribute->getErrors(), true));
            $this->Flash->error(__('The product category attribute could not be saved. Please, try again.'));
        }
        $productCategories = $this->ProductCategoryAttributes->ProductCategories->find('list', keyField: 'internal_id', valueField: 'name')->all();
        $this->set(compact('productCategoryAttribute', 'productCategories'));
    }

    /**
     * Edit method
     *
     * @param string|null $id Product Category Attribute id.
     * @return Response|null|void Redirects on successful edit, renders view otherwise.
     * @throws RecordNotFoundException When record not found.
     */
    public function edit($id = null)
    {
        $productCategoryAttribute = $this->ProductCategoryAttributes->get($id, contain: ['ProductCategoryAttributeOptions']);
        if ($this->request->is(['patch', 'post', 'put'])) {
            $postData = $this->request->getData();
            $saveOptions = [
                'associated' => ['ProductCategoryAttributeOptions'],
            ];
            Log::debug(print_r('$postData', true));
            Log::debug(print_r($postData, true));
//            if ($this->request->getData('attribute_type_id') != ProductCategoryAttributeTypeId::Constrained) {
//                $saveOptions['associated'] = [];
//                $postData['product_category_attribute_options'] = [];
//            }
            Log::debug(print_r('$postData', true));
            Log::debug(print_r($postData, true));
            $productCategoryAttribute = $this->ProductCategoryAttributes->patchEntity($productCategoryAttribute, $postData, $saveOptions);

            if ($this->ProductCategoryAttributes->save($productCategoryAttribute, $saveOptions)) {
                $this->Flash->success(__('The product category attribute has been saved.'));

                return $this->redirect(['action' => 'index']);
            }
            Log::debug('failed to save product category attribute on edit errors next');
            Log::debug(print_r('$productCategoryAttribute->getErrors()', true));
            Log::debug(print_r($productCategoryAttribute->getErrors(), true));
            $this->Flash->error(__('The product category attribute could not be saved. Please, try again.'));
        }
        $productCategories = $this->ProductCategoryAttributes->ProductCategories->find('list', limit: 200, keyField: 'internal_id', valueField: 'name')->all();
        $this->set(compact('productCategoryAttribute', 'productCategories'));
    }

    /**
     * Delete method
     *
     * @param string|null $id Product Category Attribute id.
     * @return Response|null Redirects to index.
     * @throws RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
        $this->request->allowMethod(['post', 'delete']);
        $productCategoryAttribute = $this->ProductCategoryAttributes->get($id);
        if ($this->ProductCategoryAttributes->delete($productCategoryAttribute)) {
            $this->Flash->success(__('The product category attribute has been deleted.'));
        } else {
            $this->Flash->error(__('The product category attribute could not be deleted. Please, try again.'));
        }

        return $this->redirect(['action' => 'index']);
    }
}
